
-- Default settings
local DEFAULTS = {
	minDuration = 1.5,
	fadedAlpha = 0.15,
	effectDuration = 5,
}

-- Current settings
local prefs = {}

-- Globals made upvalues
local _G = _G
local GetTime = GetTime
local next = next
local pairs = pairs

local function Anim_End(animGroup, ...)
	if animGroup.alpha then
		animGroup:GetParent():SetAlpha(animGroup.alpha)
		animGroup.alpha = nil
	end
	animGroup.cooldown:SetAlpha(1)
end

local function InitFrame(frame, cooldown)
	local animGroup = frame:CreateAnimationGroup()
	animGroup:SetScript('OnStop', Anim_End)
	animGroup:SetScript('OnFinished', Anim_End)
	animGroup.cooldown = cooldown
	frame.CF_animGroup = animGroup

	local anim = animGroup:CreateAnimation("Alpha")
	anim:SetOrder(1)
	anim:SetSmoothing("IN")
	frame.CF_anim = anim
end

local frames = setmetatable({}, { __index = function(t, cooldown)
	local frame = false
	if not cooldown.noCooldownCount then
		frame = cooldown:GetParent()
		local name = frame:GetName()
		if name and (strmatch(name, '[bB]uff%d*$') or strmatch(name, '[bB]uffIcon%d*$')) then
			frame = false
		else
			InitFrame(frame, cooldown)
		end
	end
	t[cooldown] = frame
	return frame
end})

local function updateCooldown(cooldown, start, duration)
	local frame = frames[cooldown]
	if not frame then
		return -- do not mess with auras
	end

	local animGroup = frame.CF_animGroup
	if start ~= animGroup.start and duration ~= animGroup.duration then
		animGroup.start, animGroup.duration = start, duration
		local alpha = animGroup.alpha or frame:GetAlpha()
		if animGroup:IsPlaying() then
			animGroup:Stop()
		end
		if start > 0 and duration > prefs.minDuration then
			local timeLeft = start + duration - GetTime()
			local anim = frame.CF_anim
			anim:SetDuration(min(timeLeft, prefs.effectDuration))
			anim:SetStartDelay(max(0, timeLeft - prefs.effectDuration))
			anim:SetChange(alpha * (1 - prefs.fadedAlpha))
			frame:SetAlpha(alpha * prefs.fadedAlpha)
			animGroup.alpha = alpha
			animGroup:Play()
			cooldown:SetAlpha(0)
		end
	end
end

-- Initialization and setting handling

local addonName = ...
local frame = CreateFrame("Frame")
frame:RegisterEvent('ADDON_LOADED')
frame:SetScript('OnEvent', function(self, event, arg1)
	if event == 'ADDON_LOADED' and arg1 == addonName then
		self:UnregisterEvent('ADDON_LOADED')
		self:RegisterEvent('PLAYER_LOGOUT')

		-- Load settings and apply defaults
		for k, v in pairs(DEFAULTS) do
			prefs[k] = v
		end
		if _G.CooldownFaderDB then
			for k, v in pairs(_G.CooldownFaderDB) do
				prefs[k] = v
			end
		end

		-- Create a chat command
		_G.SLASH_COOLDOWNFADER1 = "/cooldownfader"
		_G.SLASH_COOLDOWNFADER2 = "/cdf"
		_G.SlashCmdList.COOLDOWNFADER = function(arg)
			local key, value = strsplit(" ", arg or "")
			if prefs[key] then
				value = tonumber(value)
				if value then
					prefs[key] = value
				end
				print(format('|cff33ff99CooldownFader:|r %s set to %g.', key, prefs[key]))
			else
				print('|cff33ff99CooldownFader settings:|r')
				for key, value in pairs(prefs) do
					print(format('  /cooldownfader %s [value] (current: %g, default: %g).', key, value, DEFAULTS[key]))
				end
			end
		end

		-- Put our hook in place
		hooksecurefunc(getmetatable(ActionButton1Cooldown).__index, 'SetCooldown', updateCooldown)

	elseif event == 'PLAYER_LOGOUT' then
		-- Recreate the saved variable
		_G.CooldownFaderDB = nil
		for k, v in pairs(prefs) do
			if v ~= DEFAULTS[k] then
				if _G.CooldownFaderDB then
					_G.CooldownFaderDB[k] = v
				else
					_G.CooldownFaderDB = { [k] = v }
				end
			end
		end
	end
end)
